{% extends "timeseer.html" %}

{% block main %}

<div class="ts-section py-3 my-3">
    <h2 class="h4">Provide feedback</h2>
</div>

<div class="my-3 py-3">
    <a class="btn btn-primary" href="mailto:feedback@timeseer.ai">Make a wish</a>
    <a class="btn btn-primary" href="mailto:bugs@timeseer.ai">Report a bug</a>
</div>

{% endblock %}
