{% extends 'timeseer.html' %}

{% from 'macros.html' import alerter, menu_back %}

{% block menu %}
{{ menu_back(url_for('.edit_flow', flowname=flow_name, sourcename=source_name)) }}
{% endblock %}

{% block main %}
<div class="ts-section py-3 my-3">
    <h2 class="h4">Remove {{ flow_name }}</h2>
</div>

<div class="pt-3 my-3">
    {{ alerter() }}

    <form method="POST">
        <input type="hidden" name="flowname" value="{{ flow_name }}">
        <div class="btn-toolbar">
            <button type="submit" class="btn btn-primary me-2">Yes</button>
            <a class="btn btn-secondary"
               href="{{ url_for('.edit_flow', flowname=flow_name, sourcename=source_name) }}"
               role="button">
                Cancel
            </a>
        </div>
    </form>
</div>


{% endblock %}
