"""Timeseer score module that generates scores for multivariate checks."""

from typing import Any, Dict

from timeseer import ScoreInput, ScoreOutput, ScoreResult

META: Dict[str, Any] = {
    "scores": [],  # accept all checks
}


# pylint: disable=missing-function-docstring
def score(
    score_input: ScoreInput,
) -> ScoreOutput:
    return ScoreOutput(
        [
            ScoreResult(result.name, result.result)
            for result in score_input.check_results
        ]
    )
