{% extends 'series-set-template-layout.html' %}

{% from 'macros.html' import alerter, menu %}
{% from 'series-set-template-form.html' import series_sets_preview %}

{% block menu %}
{{ series_set_template_menu('Series sets') }}
{% endblock %}

{% block series_set_template_content %}

<div class="pt-3">
    {{ alerter() }}

    {% if series_sets|count > 0 %}
    {{ series_sets_preview(series_sets) }}
    {% else %}
    <div class="alert alert-info">
        No series sets found.
    </div>
    {% endif %}
</div>
{% endblock %}
